#include <stdio.h>

#include "dichotomie.h"

/*!
************************************************
* \brief Fonction de recherche d'un entier v dans 
* un tableau T de taille n. 
* Tous les entiers du tableau sont successivement
* compares a la valeur v.  
* \param T le tableau 
* \param n la taille du tableau
* \param v l'entier recherche
* \return la position dans le tableau de l'entier 
egal a v (comprise entre 0 inclu et n exclu), 
-1 sinon. 
**************************************************/    
int rechercheNaive(int T[], int n, int v)
{
  int position; //position de l'entier egal a v 
  int i;  //indice de boucle

  position = -1; 
  for (i = 0; i < n; i++)
    {
      if (T[i] == v)
	position = i; 
    }
  return position; 
}

/*!
************************************************
* \brief Fonction qui compare une fonction de 
* recherche d'un entier v dans un tableau T 
* de taille n a une fonction de recherche naive 
* dans laquelle tous les entiers du tableau 
* sont successivement compares a la valeur v.  
* \param T le tableau 
* \param n la taille du tableau
* \param v l'entier recherche
* \param fonction 
* \return 1 si les deux recherches ont donne le 
* meme resultat, 0 sinon
**************************************************/    
int unitTest(int T[], int n, int v, int (*fonction)(int*,int,int) )
{
  int i; 
  int estCorrect; //1 si la recherche a donnee le bon resultat, 0 sinon

  //affichage
  printf("Recherche de %d dans \n", v); 
  for (i=0;i<n;i++)
    {
      printf("%d ",T[i]);
    }
  printf("\n");

  //comparaison
  estCorrect = (rechercheNaive(T, 10, v) == fonction(T, 10, v)); 
  if (estCorrect)
    printf("PASSED");
  else
    printf("FAILED"); 
  printf("\n");

  return estCorrect; 
}

/*!
************************************************
* \brief Teste si la recherche d'un element dans
* un tableau trie est correcte ou non
**************************************************/    
int main()
{
  int estCorrect; //1 si la recherche a donnee le bon resultat, 0 sinon
  estCorrect = 1; 

 printf("************************ Dichotomie NON recursive *******\n");
  { //test 1
    int T[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}; 
    int v = 1; 
    estCorrect = estCorrect && unitTest(T, 10, v, dichotomie); 
  }
  { //test 2
    int T[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}; 
    int v = 10; 
    estCorrect = estCorrect && unitTest(T, 10, v, dichotomie); 
  }
  { //test 3
    int T[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}; 
    int v = 9; 
    estCorrect = estCorrect && unitTest(T, 10, v, dichotomie); 
  }
  { //test 4
    int T[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}; 
    int v = 11; 
    estCorrect = estCorrect && unitTest(T, 10, v, dichotomie); 
  }

 printf("************************ Dichotomie recursive *******\n");
  { //test 1
    int T[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}; 
    int v = 1; 
    estCorrect = estCorrect && unitTest(T, 10, v, dichotomie2); 
  }
  { //test 2
    int T[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}; 
    int v = 10; 
    estCorrect = estCorrect && unitTest(T, 10, v, dichotomie2); 
  }
  { //test 3
    int T[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}; 
    int v = 9; 
    estCorrect = estCorrect && unitTest(T, 10, v, dichotomie2); 
  }
  { //test 4
    int T[10] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}; 
    int v = 11; 
    estCorrect = estCorrect && unitTest(T, 10, v, dichotomie2); 
  }

  return estCorrect ? 0 : 1;
}
